<?php
/**
 * 2018-2022 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2022 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */


/**
 * NOTICE:
 * This module was inspired to the Prestashop module "ps_imageslider"
 * "ps_imageslider" was used as a basic code to collect and show blocks
 * Starting from its original structure, mxb_customblocks has many different features and supports css, html and "mxb_featuredproducts" "tags"
 *
 * Also its structure has been upgraded to easy duplicate and customize the module.
 * Installing 2 or more customized copies of this module, you can enable/disable it for different customers or put it into different sections of the website
 */

 /**
  * ###### DUPLICATE & CUSTOMIZE 1/4:
  *        you are customizing "mxb_customblocks_one"
  *        where "one" is your [customname]
  *        if you want to create "mxb_customblocks_products", "products" will be your [customname]
  *        [customname] in file and folder name MUST be a string without underscores and in lowercase. YES: "bestseller" - NO: "BestSeller", "best_seller", "best-seller", "bestseller22"
  *        [Customname] in Class name MUST be capitalized. YES: "Customname", "Bestseller" ....
  *        [special extensions tpl_, js_, css_, translations/*.php_]: that files are ready to be installed with the customized name of the module. You can change it as you wish but remember to leave the "_mxb_[module]_" key untouched: it will be substituted with the right module name during the installation.
  *
  *        remember to change:
  *            the module folder name to "mxb_customblocks_[customname]"
  *            the module main class filename "mxb_customblocks_[customname].php"
  *            file "config.xml":
  *                the module name value to "mxb_customblocks_[customname]"
  *            file "mxb_customblocks_[customname].php":
  *                the class name Mxb_Customblocks_[Customname]
  *                the CUSTOM_NAME const value in class declaration
  *                some values into __constuct() function
  *            file "mxb_customblocks_class.php":
  *                the class name Mxb_Customblocks_[Customname]_Class
  *                the CUSTOM_NAME const value in class declaration
  */

 /**
  * for DEBUG: if you use mxb_tools module, you'll find the log file in folder: /module/mxb_tools/log/
  *         require_once _PS_MODULE_DIR_.'mxb_tools/classes/MxbToolsDebug.php';
  * and then use this function:
  *        MxbToolsDebug::WriteLog($classname, $function, $message, $bypass_settings = false)
  *    i.e:
  *        MxbToolsDebug::WriteLog($this->name, 'className::__construct()', 'START', true);
  */

//require_once _PS_MODULE_DIR_.'mxb_tools/classes/MxbToolsDebug.php';
if (!defined('_PS_VERSION_')) {
    exit;
}

/**
 * @since   1.5.0
 */

use PrestaShop\PrestaShop\Core\Module\WidgetInterface;

$autoloadPath = __DIR__ . '/vendor/autoload.php';
if (file_exists($autoloadPath)) {
    require_once $autoloadPath;
}

require(dirname(__FILE__).'/mxb_customblocks_class.php');


 /**
  * ###### DUPLICATE & CUSTOMIZE 2/4: class declaration: change the class name as the module name: Mxb_Customblocks_[Customname]
  */
class Mxb_Customblocks_One extends Module implements WidgetInterface
{
    /* ###### DUPLICATE 3/4: set CUSTOM_NAME as "yourname" in lowercase
    *                        set $mainHook as you wish: it is the default hook for this module: displayHome will shows blocks in homepage */
    const CUSTOM_NAME     = 'one';
    public $mainHook     = 'displayHome';

    /* ###### DUPLICATE & CUSTOMIZE 4/4: go to __construct() fuction and customize last values */

    const MODULE_NAME_ROOT = 'mxb_customblocks';
    public $documentationUrl = 'https://www.mxbinformatica.it/prestashop-mxb-customblocks/';
    public $thumbnailUrl = 'https://www.mxbinformatica.it/ps_modules/thumbs/mxb_customblocks.jpg';

    // the following version variable is for development, used do update databases and other stuff without reinstalling the module
    // please check checkForUpdates() function
    // use int value !
    const MXB_DEV_VERSION = 2;

    protected $_html = '';
    protected $default_width = 779;
    protected $default_margin = 0;
    protected $default_read_more_height = 300;
    protected $default_effect_on_scroll = 1;
    protected $default_wrap = 1;
    protected $templateFile;

    public $customName = null;
    public $tableName = null;
    public $className = null;

    public $effect_array = array(
        'none',
        'mxb-transition-fade',
        'mxb-transition-from-left',
        'mxb-transition-chars-from-left',
        'mxb-transition-from-right',
        'mxb-transition-from-bottom',
        'mxb-transition-from-top',
    );


    protected function getAssociatedIdsShop($id_slide = 0) {
        $className = 'Mxb_Customblocks_' . ucfirst($this->customName) . '_Class';
        return $className::getAssociatedIdsShop((int)$id_slide);
    }
    protected function getSlide($id_slide = null) {
        $className = 'Mxb_Customblocks_' . ucfirst($this->customName) . '_Class';
        if ($id_slide !== null) {
            return new $className((int)$id_slide);
        } else {
            return new $className();
        }
    }

    public function __construct()
    {
        //MxbToolsDebug::WriteLog($this->name, 'Mxb_Customblocks_One::__construct()', 'START', true);
        /* ###### DUPLICATE & CUSTOMIZE 4/4: following values could be parametrized but the Prestashop module trust verification need simple strings to verify the module... */
        // PS module verification doesn't allow to construct name with other variables
        $this->name = 'mxb_customblocks_one';
        // PS module verification need a displayName identical to the one into config.xml
        $this->displayName = 'MXB Custom Blocks: one';

        /* ###### DUPLICATE & CUSTOMIZE: no more changes are needed from here */

        $this->customName = self::CUSTOM_NAME;
        $this->rootName = self::MODULE_NAME_ROOT . '_';
        //$this->name = $this->rootName . $this->customName;
        //$this->displayName = 'MXB Custom Blocks: ' . $this->customName;
        $this->description = $this->l('Adds paragraphs to your pages.');
        $this->version = '1.0.4';
        $this->dev_version = self::MXB_DEV_VERSION;

        $this->tab = 'front_office_features';
        $this->author = 'MXB Informatica';
        $this->need_instance = 0;
        $this->secure_key = Tools::encrypt($this->name);
        $this->bootstrap = true;
        $this->ps_versions_compliancy = array('min' => '1.7.4.0', 'max' => _PS_VERSION_);

        $this->tableName = $this->rootName . $this->customName;
        $this->className = 'Mxb_Customblocks_' . ucfirst($this->customName) . '_Class';
        $this->customCssFolder = "customcss";
        $this->templateFile = 'module:' . $this->name . '/views/templates/hook/slider.tpl';

        // configuration values names
        $this->CUSTOMBLOCKS_MARGIN = $this->name . '_MARGIN';
        $this->CUSTOMBLOCKS_EFFECT_ON_SCROLL = $this->name . '_EFFECT_ON_SCROLL';
        $this->CUSTOMBLOCKS_READ_MORE_HEIGHT = $this->name . '_READ_MORE_HEIGHT';

        // CSS management
        $this->CUSTOMBLOCKS_CUSTOM_CSS_CONTENT = $this->name . '_CUSCSS';
        $this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE = $this->name . '_CUSCSS_FLAG';
        $this->CUSTOMBLOCKS_SHUFFLE_ENABLE = $this->name . '_SHUFFLE_FLAG';

        parent::__construct();
        
        $this->checkForUpdates();

        // REMOVE IN PRODUCTION MODE: set module files JS, TPL, CSS
        //$this->installModuleFiles();
    }

    /**
     * @see Module::install()
     */
    public function install()
    {
        /* Adds Module */
        if (parent::install() &&
            $this->registerHook('displayHeader') &&
            $this->registerHook($this->mainHook) &&
            $this->registerHook('actionShopDataDuplication') &&
            $this->registerHook('actionAdminControllerSetMedia')
        ) {
            $shops = Shop::getContextListShopID();
            $shop_groups_list = array();

            /* Setup each shop */
            foreach ($shops as $shop_id) {
                $shop_group_id = (int)Shop::getGroupFromShop($shop_id, true);

                if (!in_array($shop_group_id, $shop_groups_list)) {
                    $shop_groups_list[] = $shop_group_id;
                }

                /* Sets up configuration */
                $res = Configuration::updateValue($this->CUSTOMBLOCKS_MARGIN, $this->default_margin, false, $shop_group_id, $shop_id);
                $res &= Configuration::updateValue($this->CUSTOMBLOCKS_EFFECT_ON_SCROLL, $this->default_effect_on_scroll, false, $shop_group_id, $shop_id);
                $res &= Configuration::updateValue($this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE, 0, false, $shop_group_id, $shop_id);
                $res &= Configuration::updateValue($this->CUSTOMBLOCKS_SHUFFLE_ENABLE, 0, false, $shop_group_id, $shop_id);
                $res &= Configuration::updateValue($this->CUSTOMBLOCKS_READ_MORE_HEIGHT, $this->default_read_more_height, false, $shop_group_id, $shop_id);
                

            }

            /* Sets up Shop Group configuration */
            if (count($shop_groups_list)) {
                foreach ($shop_groups_list as $shop_group_id) {
                    $res &= Configuration::updateValue($this->CUSTOMBLOCKS_MARGIN, $this->default_margin, false, $shop_group_id);
                    $res &= Configuration::updateValue($this->CUSTOMBLOCKS_EFFECT_ON_SCROLL, $this->default_effect_on_scroll, false, $shop_group_id);
                    $res &= Configuration::updateValue($this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE, 0, false, $shop_group_id);
                    $res &= Configuration::updateValue($this->CUSTOMBLOCKS_SHUFFLE_ENABLE, 0, false, $shop_group_id);
                    $res &= Configuration::updateValue($this->CUSTOMBLOCKS_READ_MORE_HEIGHT, $this->default_read_more_height, false, $shop_group_id);
                }
            }

            /* Sets up Global configuration */
            $res &= Configuration::updateValue($this->CUSTOMBLOCKS_MARGIN, $this->default_margin);
            $res &= Configuration::updateValue($this->CUSTOMBLOCKS_EFFECT_ON_SCROLL, $this->default_effect_on_scroll);
            $res &= Configuration::updateValue($this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE, 0);
            $res &= Configuration::updateValue($this->CUSTOMBLOCKS_SHUFFLE_ENABLE, 0);
            $res &= Configuration::updateValue($this->CUSTOMBLOCKS_READ_MORE_HEIGHT, $this->default_read_more_height);

            /* Creates tables */
            $res &= $this->createTables();

            /* Adds samples */
            if ($res) {
                $this->installSamples();
            }

            // Disable on mobiles and tablets
            //$this->disableDevice(Context::DEVICE_MOBILE);

            // set module files JS, TPL, CSS
            $this->installModuleFiles();

            return (bool)$res;
        }

        return false;
    }

    /**
     * Adds samples
     */
    protected function installSamples()
    {
        $languages = Language::getLanguages(false);
        for ($i = 1; $i <= 2; ++$i) {
            $slide = $this->getSlide();
            $slide->position = $i;
            $slide->active = 1;
            $slide->fixed_bg = 1;
            $slide->repeat_bg = 0;
            $slide->custom_class = '';
            $slide->id_cms = 0;

            foreach ($languages as $language) {
                $slide->title[$language['id_lang']] = 'Sample '.$i;
                $slide->description[$language['id_lang']] = '<h3>EXCEPTEUR OCCAECAT</h3>'
                    . '<p>Lorem ipsum dolor sit amet, consectetur adipiscing elit. '
                    . 'Proin tristique in tortor et dignissim. Quisque non tempor leo. '
                    . 'Maecenas egestas sem elit</p>';
                $slide->legend[$language['id_lang']] = 'sample-'.$i;
                $slide->url[$language['id_lang']] = 'https://www.mxbinformatica.it/prestashop-mxb-customblocks/';
                    //.'&utm_campaign=back-office-'.Tools::strtoupper($this->context->language->iso_code)
                    //.'&utm_content='.(defined('_PS_HOST_MODE_') ? 'ondemand' : 'download');
                //$slide->image[$language['id_lang']] = 'sample-'.$i.'.jpg';
                $slide->id_transition = 1;
            }
            $slide->add();
        }
    }

    /**
     * @see Module::uninstall()
     */
    public function uninstall()
    {
        /* Deletes Module */
        if (parent::uninstall()) {
            /* Deletes tables */
            $res = $this->deleteTables();

            /* Unsets configuration */
            $res &= Configuration::deleteByName($this->CUSTOMBLOCKS_MARGIN);
            $res &= Configuration::deleteByName($this->CUSTOMBLOCKS_EFFECT_ON_SCROLL);
            $res &= Configuration::deleteByName($this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE);
            $res &= Configuration::deleteByName($this->CUSTOMBLOCKS_SHUFFLE_ENABLE);
            $res &= Configuration::deleteByName($this->CUSTOMBLOCKS_READ_MORE_HEIGHT);
            $res &= Configuration::deleteByName($this->name . '_VERSION');
            return (bool)$res;
        }

        return false;
    }

    /**
     * Creates tables
     */
    protected function createTables()
    {
        /* Slides */
        $res = (bool)Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.$this->tableName  . '`(
                `id_'.$this->tableName.'_slides` int(10) unsigned NOT NULL AUTO_INCREMENT,
                `id_shop` int(10) unsigned NOT NULL,
                PRIMARY KEY (`id_'.$this->tableName.'_slides`, `id_shop`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');

        /* Slides configuration */
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.$this->tableName  . '_slides` (
              `id_'.$this->tableName.'_slides` int(10) unsigned NOT NULL AUTO_INCREMENT,
              `position` int(10) unsigned NOT NULL DEFAULT \'0\',
              `active` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
              `fixed_bg` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
              `repeat_bg` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
              `id_transition` tinyint(1) unsigned NOT NULL DEFAULT \'0\',
              `custom_class` varchar(255) NOT NULL,
              `id_cms` int(10) unsigned NOT NULL DEFAULT \'0\',
              PRIMARY KEY (`id_'.$this->tableName.'_slides`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');

        /* Slides lang configuration */
        $res &= Db::getInstance()->execute('
            CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.$this->tableName  . '_slides_lang` (
              `id_'.$this->tableName.'_slides` int(10) unsigned NOT NULL,
              `id_lang` int(10) unsigned NOT NULL,
              `title` varchar(255) NOT NULL,
              `description` text NOT NULL,
              `legend` varchar(255) NOT NULL,
              `url` varchar(255) NOT NULL,
              `image` varchar(255) NOT NULL,
              PRIMARY KEY (`id_'.$this->tableName.'_slides`,`id_lang`)
            ) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=UTF8;
        ');

        return $res;
    }

    /**
     * deletes tables
     */
    protected function deleteTables()
    {
        $slides = $this->getSlides();
        foreach ($slides as $slide) {
            $to_del = $this->getSlide($slide['id_slide']);
            $to_del->delete();
        }

        return Db::getInstance()->execute('
            DROP TABLE IF EXISTS `'._DB_PREFIX_.$this->tableName  . '`, `'._DB_PREFIX_.$this->tableName  . '_slides`, `'._DB_PREFIX_.$this->tableName  . '_slides_lang`;
        ');
    }

    public function getContent()
    {
        // update css
        //$this->checkModuleFiles();
        //$this->registerHook('actionAdminControllerSetMedia');

        $this->_html .= $this->headerHTML();

        /* Validate & process */
        if (Tools::isSubmit('submitSlide') || Tools::isSubmit('delete_id_slide') ||
            Tools::isSubmit('submitSlider') ||
            Tools::isSubmit('changeStatus')
        ) {
            if ($this->_postValidation()) {
                $this->_postProcess();
                $this->_html .= $this->renderForm();
                $this->_html .= $this->renderList();
            } else {
                $this->_html .= $this->renderAddForm();
            }
            $this->clearCache();
        } elseif (Tools::isSubmit('addSlide') || (Tools::isSubmit('id_slide') && $this->slideExists((int)Tools::getValue('id_slide')))) {
            if (Tools::isSubmit('addSlide')) {
                $mode = 'add';
            } else {
                $mode = 'edit';
            }

            if ($mode == 'add') {
                if (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL) {
                    $this->_html .= $this->renderAddForm();
                } else {
                    $this->_html .= $this->getShopContextError(null, $mode);
                }
            } else {
                $associated_shop_ids = $this->getAssociatedIdsShop((int)Tools::getValue('id_slide'));
                $context_shop_id = (int)Shop::getContextShopID();

                if ($associated_shop_ids === false) {
                    $this->_html .= $this->getShopAssociationError((int)Tools::getValue('id_slide'));
                } elseif (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL && in_array($context_shop_id, $associated_shop_ids)) {
                    if (count($associated_shop_ids) > 1) {
                        $this->_html = $this->getSharedSlideWarning();
                    }
                    $this->_html .= $this->renderAddForm();
                } else {
                    $shops_name_list = array();
                    foreach ($associated_shop_ids as $shop_id) {
                        $associated_shop = new Shop((int)$shop_id);
                        $shops_name_list[] = $associated_shop->name;
                    }
                    $this->_html .= $this->getShopContextError($shops_name_list, $mode);
                }
            }
        } else {

            $this->_html .= $this->getWarningMultishopHtml().$this->getCurrentShopInfoMsg().$this->renderForm();

            if (Shop::getContext() != Shop::CONTEXT_GROUP && Shop::getContext() != Shop::CONTEXT_ALL) {
                $this->_html .= $this->renderList();
            }
        }

        return $this->_html;
    }

    protected function _postValidation()
    {
        $errors = array();

        /* Validation for Slider configuration */
        if (Tools::isSubmit('submitSlider')) {
            if (!Validate::isInt(Tools::getValue('CUSTOMBLOCKS_MARGIN'))) {
                $errors[] = $this->l('Invalid values');
            }
            if (!Validate::isInt(Tools::getValue('CUSTOMBLOCKS_READ_MORE_HEIGHT'))) {
                $errors[] = $this->l('Invalid values');
            }
        } elseif (Tools::isSubmit('changeStatus')) {
            if (!Validate::isInt(Tools::getValue('id_slide'))) {
                $errors[] = $this->l('Invalid block');
            }
        } elseif (Tools::isSubmit('submitSlide')) {
            /* Checks state (active) */
            if (!Validate::isInt(Tools::getValue('active_slide')) || (Tools::getValue('active_slide') != 0 && Tools::getValue('active_slide') != 1)) {
                $errors[] = $this->l('Invalid block state.');
            }
            /* Checks state (fixed_bg) */
            if (!Validate::isInt(Tools::getValue('fixed_bg_slide')) || (Tools::getValue('fixed_bg_slide') != 0 && Tools::getValue('fixed_bg_slide') != 1)) {
                $errors[] = $this->l('Invalid block bg state.');
            }
            /* Checks state (repeat_bg) */
            if (!Validate::isInt(Tools::getValue('repeat_bg_slide')) || (Tools::getValue('repeat_bg_slide') != 0 && Tools::getValue('repeat_bg_slide') != 1)) {
                $errors[] = $this->l('Invalid block bg state.');
            }
            /* Checks state (id_transition) */
            if (!Validate::isInt(Tools::getValue('id_transition_slide'))) {
                $errors[] = $this->l('Invalid block id_transition.');
            }

            /* Checks position
            if (!Validate::isInt(Tools::getValue('position')) || (Tools::getValue('position') < 0)) {
                $errors[] = $this->l('Invalid block position.');
            }*/
            /* If edit : checks id_slide */
            if (Tools::isSubmit('id_slide')) {
                if (!Validate::isInt(Tools::getValue('id_slide')) && !$this->slideExists(Tools::getValue('id_slide'))) {
                    $errors[] = $this->l('Invalid block ID');
                }
            }
            /* Checks title/url/legend/description/image */
            $languages = Language::getLanguages(false);
            foreach ($languages as $language) {
                if (Tools::strlen(Tools::getValue('title_' . $language['id_lang'])) > 255) {
                    $errors[] = $this->l('The title is too long.');
                }
                if (Tools::strlen(Tools::getValue('legend_' . $language['id_lang'])) > 255) {
                    $errors[] = $this->l('The caption is too long.');
                }
                if (Tools::strlen(Tools::getValue('url_' . $language['id_lang'])) > 255) {
                    $errors[] = $this->l('The URL is too long.');
                }
                if (Tools::strlen(Tools::getValue('description_' . $language['id_lang'])) > 4000) {
                    $errors[] = $this->l('The description is too long.');
                }
                if (Tools::strlen(Tools::getValue('url_' . $language['id_lang'])) > 0 && !Validate::isUrl(Tools::getValue('url_' . $language['id_lang']))) {
                    $errors[] = $this->l('The URL format is not correct.');
                }
                if (Tools::getValue('image_' . $language['id_lang']) != null && !Validate::isFileName(Tools::getValue('image_' . $language['id_lang']))) {
                    $errors[] = $this->l('Invalid filename.');
                }
                if (Tools::getValue('image_old_' . $language['id_lang']) != null && !Validate::isFileName(Tools::getValue('image_old_' . $language['id_lang']))) {
                    $errors[] = $this->l('Invalid filename.');
                }
            }

            /* Checks title/url/legend/description for default lang */
            $id_lang_default = (int)Configuration::get('PS_LANG_DEFAULT');
            //if (Tools::strlen(Tools::getValue('url_' . $id_lang_default)) == 0) {
            //    $errors[] = $this->l('The URL is not set.');
            //}
            //if (!Tools::isSubmit('has_picture') && (!isset($_FILES['image_' . $id_lang_default]) || empty($_FILES['image_' . $id_lang_default]['tmp_name']))) {
            //    $errors[] = $this->l('The image is not set.');
            //}
            if (Tools::getValue('image_old_'.$id_lang_default) && !Validate::isFileName(Tools::getValue('image_old_'.$id_lang_default))) {
                $errors[] = $this->l('The image is not set.');
            }
        } elseif (Tools::isSubmit('delete_id_slide') && (!Validate::isInt(Tools::getValue('delete_id_slide')) || !$this->slideExists((int)Tools::getValue('delete_id_slide')))) {
            $errors[] = $this->l('Invalid block ID');
        }

        /* Display errors if needed */
        if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));

            return false;
        }

        /* Returns if validation is ok */

        return true;
    }

    protected function _postProcess()
    {
        $errors = array();
        $shop_context = Shop::getContext();

        /* Processes Slider */
        if (Tools::isSubmit('submitSlider')) {
            $shop_groups_list = array();
            $shops = Shop::getContextListShopID();

            foreach ($shops as $shop_id) {
                $shop_group_id = (int)Shop::getGroupFromShop($shop_id, true);

                if (!in_array($shop_group_id, $shop_groups_list)) {
                    $shop_groups_list[] = $shop_group_id;
                }

                $res = Configuration::updateValue($this->CUSTOMBLOCKS_MARGIN, (int)Tools::getValue($this->CUSTOMBLOCKS_MARGIN), false, $shop_group_id, $shop_id);
                $res &= Configuration::updateValue($this->CUSTOMBLOCKS_EFFECT_ON_SCROLL, (int)Tools::getValue($this->CUSTOMBLOCKS_EFFECT_ON_SCROLL), false, $shop_group_id, $shop_id);
                $res &= Configuration::updateValue($this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE, (int)Tools::getValue($this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE), false, $shop_group_id, $shop_id);
                $res &= Configuration::updateValue($this->CUSTOMBLOCKS_SHUFFLE_ENABLE, (int)Tools::getValue($this->CUSTOMBLOCKS_SHUFFLE_ENABLE), false, $shop_group_id, $shop_id);
                $res &= Configuration::updateValue($this->CUSTOMBLOCKS_READ_MORE_HEIGHT, (int)Tools::getValue($this->CUSTOMBLOCKS_READ_MORE_HEIGHT), false, $shop_group_id, $shop_id);

                   // save custom css
                $this->saveCssFile(Tools::getValue($this->CUSTOMBLOCKS_CUSTOM_CSS_CONTENT));

            }

            /* Update global shop context if needed*/
            switch ($shop_context) {
                case Shop::CONTEXT_ALL:
                    $res &= Configuration::updateValue($this->CUSTOMBLOCKS_MARGIN, (int)Tools::getValue($this->CUSTOMBLOCKS_MARGIN));
                    $res &= Configuration::updateValue($this->CUSTOMBLOCKS_EFFECT_ON_SCROLL, (int)Tools::getValue($this->CUSTOMBLOCKS_EFFECT_ON_SCROLL));
                    $res &= Configuration::updateValue($this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE, (int)Tools::getValue($this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE));
                    $res &= Configuration::updateValue($this->CUSTOMBLOCKS_SHUFFLE_ENABLE, (int)Tools::getValue($this->CUSTOMBLOCKS_SHUFFLE_ENABLE));
                    $res &= Configuration::updateValue($this->CUSTOMBLOCKS_READ_MORE_HEIGHT, (int)Tools::getValue($this->CUSTOMBLOCKS_READ_MORE_HEIGHT));

                    if (count($shop_groups_list)) {
                        foreach ($shop_groups_list as $shop_group_id) {
                            $res &= Configuration::updateValue($this->CUSTOMBLOCKS_MARGIN, (int)Tools::getValue($this->CUSTOMBLOCKS_MARGIN), false, $shop_group_id);
                            $res &= Configuration::updateValue($this->CUSTOMBLOCKS_EFFECT_ON_SCROLL, (int)Tools::getValue($this->CUSTOMBLOCKS_EFFECT_ON_SCROLL), false, $shop_group_id);
                            $res &= Configuration::updateValue($this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE, (int)Tools::getValue($this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE), false, $shop_group_id);
                            $res &= Configuration::updateValue($this->CUSTOMBLOCKS_SHUFFLE_ENABLE, (int)Tools::getValue($this->CUSTOMBLOCKS_SHUFFLE_ENABLE), false, $shop_group_id);
                            $res &= Configuration::updateValue($this->CUSTOMBLOCKS_READ_MORE_HEIGHT, (int)Tools::getValue($this->CUSTOMBLOCKS_READ_MORE_HEIGHT), false, $shop_group_id);
                        }
                    }
                    break;
                case Shop::CONTEXT_GROUP:
                    if (count($shop_groups_list)) {
                        foreach ($shop_groups_list as $shop_group_id) {
                            $res &= Configuration::updateValue($this->CUSTOMBLOCKS_MARGIN, (int)Tools::getValue($this->CUSTOMBLOCKS_MARGIN), false, $shop_group_id);
                            $res &= Configuration::updateValue($this->CUSTOMBLOCKS_EFFECT_ON_SCROLL, (int)Tools::getValue($this->CUSTOMBLOCKS_EFFECT_ON_SCROLL), false, $shop_group_id);
                            $res &= Configuration::updateValue($this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE, (int)Tools::getValue($this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE), false, $shop_group_id);
                            $res &= Configuration::updateValue($this->CUSTOMBLOCKS_SHUFFLE_ENABLE, (int)Tools::getValue($this->CUSTOMBLOCKS_SHUFFLE_ENABLE), false, $shop_group_id);
                            $res &= Configuration::updateValue($this->CUSTOMBLOCKS_READ_MORE_HEIGHT, (int)Tools::getValue($this->CUSTOMBLOCKS_READ_MORE_HEIGHT), false, $shop_group_id);
                        }
                    }
                    break;
            }

            $this->clearCache();

            if (!$res) {
                $errors[] = $this->displayError($this->l('The configuration could not be updated.'));
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=6&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
            }
        } elseif (Tools::isSubmit('changeStatus') && Tools::isSubmit('id_slide')) {
            $slide = $this->getSlide((int)Tools::getValue('id_slide'));
            if ($slide->active == 0) {
                $slide->active = 1;
            } else {
                $slide->active = 0;
            }
            $res = $slide->update();
            $this->clearCache();
            $this->_html .= ($res ? $this->displayConfirmation($this->getTranslator()->trans('Configuration updated', array(), 'Admin.Notifications.Success')) : $this->displayError($this->l('The configuration could not be updated.')));
        } elseif (Tools::isSubmit('submitSlide')) {
            /* Sets ID if needed */
            if (Tools::getValue('id_slide')) {
                $slide = $this->getSlide((int)Tools::getValue('id_slide'));
                if (!Validate::isLoadedObject($slide)) {
                    $this->_html .= $this->displayError($this->l('Invalid block ID'));
                    return false;
                }
            } else {
                $slide = $this->getSlide();
                /* Sets position */
                $slide->position = (int)$this->getNextPosition(); // $slide->position = (int)Tools::getValue('position'); (outside the if/else
            }
            /* Sets active */
            $slide->active = (int)Tools::getValue('active_slide');

            $slide->fixed_bg = (int)Tools::getValue('fixed_bg_slide');
            $slide->repeat_bg = (int)Tools::getValue('repeat_bg_slide');
            $slide->id_transition = (int)Tools::getValue('id_transition_slide');

            $slide->custom_class = (string)Tools::getValue('custom_class_slide');
            $slide->id_cms = (string)Tools::getValue('id_cms_slide');

            /* Sets each langue fields */
            $languages = Language::getLanguages(false);

            foreach ($languages as $language) {
                $slide->title[$language['id_lang']] = Tools::getValue('title_'.$language['id_lang']);
                $slide->url[$language['id_lang']] = Tools::getValue('url_'.$language['id_lang']);
                $slide->legend[$language['id_lang']] = Tools::getValue('legend_'.$language['id_lang']);
                $slide->description[$language['id_lang']] = Tools::getValue('description_'.$language['id_lang']);

                /* Uploads image and sets slide */
                $type = Tools::strtolower(Tools::substr(strrchr($_FILES['image_'.$language['id_lang']]['name'], '.'), 1));
                $imagesize = @getimagesize($_FILES['image_'.$language['id_lang']]['tmp_name']);
                if (isset($_FILES['image_'.$language['id_lang']]) &&
                    isset($_FILES['image_'.$language['id_lang']]['tmp_name']) &&
                    !empty($_FILES['image_'.$language['id_lang']]['tmp_name']) &&
                    !empty($imagesize) &&
                    in_array(
                        Tools::strtolower(Tools::substr(strrchr($imagesize['mime'], '/'), 1)), array(
                            'jpg',
                            'gif',
                            'jpeg',
                            'png'
                        )
                    ) &&
                    in_array($type, array('jpg', 'gif', 'jpeg', 'png'))
                ) {
                    $temp_name = tempnam(_PS_TMP_IMG_DIR_, 'PS');
                    $salt = sha1(microtime());
                    if ($error = ImageManager::validateUpload($_FILES['image_'.$language['id_lang']])) {
                        $errors[] = $error;
                    } elseif (!$temp_name || !move_uploaded_file($_FILES['image_'.$language['id_lang']]['tmp_name'], $temp_name)) {
                        return false;
                    } elseif (!ImageManager::resize($temp_name, dirname(__FILE__).'/images/'.$salt.'_'.$_FILES['image_'.$language['id_lang']]['name'], null, null, $type)) {
                        $errors[] = $this->displayError($this->getTranslator()->trans('An error occurred during the image upload process.', array(), 'Admin.Notifications.Error'));
                    }
                    if (isset($temp_name)) {
                        @unlink($temp_name);
                    }
                    $slide->image[$language['id_lang']] = $salt.'_'.$_FILES['image_'.$language['id_lang']]['name'];
                } elseif (Tools::getValue('image_old_'.$language['id_lang']) != '') {
                    $slide->image[$language['id_lang']] = Tools::getValue('image_old_' . $language['id_lang']);
                } elseif ((int)Tools::getValue('remove_image_slide') === 1) {
                    // remove old image (TODO) and set the default empty image
                    $slide->image[$language['id_lang']] = '';

                }
            }

            /* Processes if no errors  */
            if (!$errors) {
                /* Adds */
                if (!Tools::getValue('id_slide')) {
                    if (!$slide->add()) {
                        $errors[] = $this->displayError($this->l('The block could not be added.'));
                    }
                } elseif (!$slide->update()) {
                    $errors[] = $this->displayError($this->l('The block could not be updated.'));
                }
                $this->clearCache();
            }
        } elseif (Tools::isSubmit('delete_id_slide')) {
            $slide = $this->getSlide((int)Tools::getValue('delete_id_slide'));
            $res = $slide->delete();
            $this->clearCache();
            if (!$res) {
                $this->_html .= $this->displayError('Could not delete.');
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=1&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
            }
        }

        /* Display errors if needed */
        if (count($errors)) {
            $this->_html .= $this->displayError(implode('<br />', $errors));
        } elseif (Tools::isSubmit('submitSlide') && Tools::getValue('id_slide')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        } elseif (Tools::isSubmit('submitSlide')) {
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        }
    }

    public function hookdisplayHeader($params)
    {
        $this->context->controller->registerStylesheet('modules-customblocks', 'modules/'.$this->name.'/views/css/front.css', ['media' => 'all', 'priority' => 150]);
        //$this->context->controller->registerJavascript('modules-responsiveslides', 'modules/'.$this->name.'/views/js/responsiveslides.min.js', ['position' => 'bottom', 'priority' => 150]);
        $this->context->controller->registerJavascript('modules-customblocks'.$this->customName, 'modules/'.$this->name.'/views/js/front.js', ['position' => 'bottom', 'priority' => 150]);

         if ((bool) Configuration::get($this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE)) {
            $this->context->controller->addCSS($this->_path.'/views'.DIRECTORY_SEPARATOR.$this->customCssFolder.DIRECTORY_SEPARATOR .'/custom.css');
         }
    }
    
    public function hookActionAdminControllerSetMedia()
    {
        $this->context->controller->addJS($this->_path.'views/js/back.js');
        //$this->context->controller->addCSS($this->_path.'views/css/back.css');
        //$this->context->controller->addJqueryUI('ui.datepicker');
    }

    public function renderWidget($hookName = null, array $configuration = [])
    {
        if (!$this->isCached($this->templateFile, $this->getCacheId())) {
            $this->smarty->assign($this->getWidgetVariables($hookName, $configuration));
        }

        return $this->fetch($this->templateFile, $this->getCacheId());
    }

    public function getWidgetVariables($hookName = null, array $configuration = [])
    {
        $slides = $this->getSlides(true);
        if (is_array($slides)) {
            foreach ($slides as &$slide) {
                if ( $slide['image'] && strlen($slide['image']) ) {
                    if ((bool)$slide['repeat_bg']) {
                        $slide['size'] = 'width="100%"';
                        $slide['css_size'] = 'width:100%;';
                    }
                    else {
                        $slide['sizes'] = @getimagesize((dirname(__FILE__) . DIRECTORY_SEPARATOR . 'images' . DIRECTORY_SEPARATOR . $slide['image']));
                        //MxbToolsDebug::WriteLog($this->name, 'getWidgetVariables()', 'slide: ' . print_r($slide, true));

                        if (isset($slide['sizes'][3]) && $slide['sizes'][3]) {
                            $slide['size'] = $slide['sizes'][3];
                            $slide['css_size'] = 'width:'.$slide['sizes'][0].'px;height:'.$slide['sizes'][1].'px;';
                        }
                    }
                }

                // CMS
                $idCms = (int)$slide['id_cms'];
                if ($idCms > 0) {
                    $cms_content = CMS::getCMSContent($idCms, $this->context->language->id, $this->context->shop->id);
                    //MxbToolsDebug::WriteLog($this->name, 'Mxb_Customblocks_One::getWidgetVariables()', 'cms_content='.print_r($cms_content, true), true);
                    if (isset($cms_content['content'])) {
                        $slide['cms_content'] = $cms_content['content'];
                    }
                }
                
                // read-more option
                if (strpos($slide['custom_class'], 'read-more') !== false) {
                    $slide['read_more'] = 1;
                }
            }
        }
        $config = $this->getConfigFieldsValues();

        return [
            'customblocks' => [
                'margin' => $config[$this->CUSTOMBLOCKS_MARGIN],
                'read_more_max_height' => $config[$this->CUSTOMBLOCKS_READ_MORE_HEIGHT],
                'effect_on_scroll' => $config[$this->CUSTOMBLOCKS_EFFECT_ON_SCROLL] ? 1 : 0,
                'enable_css' => $config[$this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE] ? 'true' : 'false',
                'shuffle' => $config[$this->CUSTOMBLOCKS_SHUFFLE_ENABLE] ? 'true' : 'false',
                'slides' => $slides,
                'read_more_open_text' => $this->l('Show all'),
                'read_more_close_text' => $this->l('Hide details'),
                'name' => $this->customName,
            ],
        ];
    }

    protected function updateUrl($link)
    {
        if (substr($link, 0, 7) !== "http://" && substr($link, 0, 8) !== "https://") {
            $link = "https://" . $link;
        }

        return $link;
    }

    public function clearCache()
    {
        $this->_clearCache($this->templateFile);
    }

    public function hookActionShopDataDuplication($params)
    {
        Db::getInstance()->execute('
            INSERT IGNORE INTO '._DB_PREFIX_.$this->tableName  . ' (id_'.$this->tableName.'_slides, id_shop)
            SELECT id_'.$this->tableName.'_slides, '.(int)$params['new_id_shop'].'
            FROM '._DB_PREFIX_.$this->tableName  . '
            WHERE id_shop = '.(int)$params['old_id_shop']
        );
        $this->clearCache();
    }

    public function headerHTML()
    {
        if (Tools::getValue('controller') != 'AdminModules' && Tools::getValue('configure') != $this->name) {
            return;
        }

        $this->context->controller->addJqueryUI('ui.sortable');
        /* Style & js for fieldset 'slides configuration' */
        $html = '<script type="text/javascript">
            $(function() {
                var $mySlides = $("#slides");
                $mySlides.sortable({
                    opacity: 0.6,
                    cursor: "move",
                    update: function() {
                        var order = $(this).sortable("serialize") + "&action=updateSlidesPosition";
                        $.post("'.$this->context->shop->physical_uri.$this->context->shop->virtual_uri.'modules/'.$this->name.'/ajax.php?secure_key='.$this->secure_key.'", order);
                        }
                    });
                $mySlides.hover(function() {
                    $(this).css("cursor","move");
                    },
                    function() {
                    $(this).css("cursor","auto");
                });
            });
        </script>';

        return $html;
    }

    public function getNextPosition()
    {
        $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow('
            SELECT MAX(hss.`position`) AS `next_position`
            FROM `'._DB_PREFIX_.$this->tableName  . '_slides` hss, `'._DB_PREFIX_.$this->tableName  . '` hs
            WHERE hss.`id_'.$this->tableName.'_slides` = hs.`id_'.$this->tableName.'_slides` AND hs.`id_shop` = '.(int)$this->context->shop->id
        );

        return (++$row['next_position']);
    }

    public function getSlides($active = null)
    {
        $this->context = Context::getContext();
        $id_shop = $this->context->shop->id;
        $id_lang = $this->context->language->id;

        $slides = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
            SELECT hs.`id_'.$this->tableName.'_slides` as id_slide, hss.`position`, hss.`active`, hssl.`title`,
            hssl.`url`, hssl.`legend`, hssl.`description`, hssl.`image`, hss.`fixed_bg`, hss.`repeat_bg`, hss.`id_transition`, hss.`custom_class`, hss.`id_cms`
            FROM '._DB_PREFIX_.$this->tableName  . ' hs
            LEFT JOIN '._DB_PREFIX_.$this->tableName  . '_slides hss ON (hs.id_'.$this->tableName.'_slides = hss.id_'.$this->tableName.'_slides)
            LEFT JOIN '._DB_PREFIX_.$this->tableName  . '_slides_lang hssl ON (hss.id_'.$this->tableName.'_slides = hssl.id_'.$this->tableName.'_slides)
            WHERE id_shop = '.(int)$id_shop.'
            AND hssl.id_lang = '.(int)$id_lang.
            ($active ? ' AND hss.`active` = 1' : ' ').'
            ORDER BY hss.position'
        );


        foreach ($slides as &$slide) {
            $slide['image_url'] = ( $slide['image'] && strlen($slide['image']) > 0 ? $this->context->link->getMediaLink(_MODULE_DIR_.$this->name . '/images/'.$slide['image']) : null );
            $slide['url'] = ( $slide['url'] && strlen($slide['url']) > 0 ? $this->updateUrl($slide['url']) : null );
            $slide['transition'] = $this->effect_array[(int)$slide['id_transition']];
        }


        $id_shop_group = Shop::getContextShopGroupID();
        $id_shop = Shop::getContextShopID();
        $shuffle = (bool)Configuration::get($this->CUSTOMBLOCKS_SHUFFLE_ENABLE, null, $id_shop_group, $id_shop);
        if ($shuffle)
            shuffle($slides);


        return $slides;
    }

    public function getAllImagesBySlidesId($id_slides, $active = null, $id_shop = null)
    {
        $this->context = Context::getContext();
        $images = array();

        if (!isset($id_shop))
            $id_shop = $this->context->shop->id;

        $results = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
            SELECT hssl.`image`, hssl.`id_lang`
            FROM '._DB_PREFIX_.$this->tableName  . ' hs
            LEFT JOIN '._DB_PREFIX_.$this->tableName  . '_slides hss ON (hs.id_'.$this->tableName.'_slides = hss.id_'.$this->tableName.'_slides)
            LEFT JOIN '._DB_PREFIX_.$this->tableName  . '_slides_lang hssl ON (hss.id_'.$this->tableName.'_slides = hssl.id_'.$this->tableName.'_slides)
            WHERE hs.`id_'.$this->tableName.'_slides` = '.(int)$id_slides.' AND hs.`id_shop` = '.(int)$id_shop.
            ($active ? ' AND hss.`active` = 1' : ' ')
        );

        foreach ($results as $result)
            $images[$result['id_lang']] = $result['image'];

        return $images;
    }

    public function displayStatus($id_slide, $active)
    {
        $title = ((int)$active == 0 ? $this->getTranslator()->trans('Disabled', array(), 'Admin.Global') : $this->getTranslator()->trans('Enabled', array(), 'Admin.Global'));
        $icon = ((int)$active == 0 ? 'icon-remove' : 'icon-check');
        $class = ((int)$active == 0 ? 'btn-danger' : 'btn-success');
        $html = '<a class="btn '.$class.'" href="'.AdminController::$currentIndex.
            '&configure='.$this->name.
                '&token='.Tools::getAdminTokenLite('AdminModules').
                '&changeStatus&id_slide='.(int)$id_slide.'" title="'.$title.'"><i class="'.$icon.'"></i> '.$title.'</a>';

        return $html;
    }

    public function slideExists($id_slide)
    {
        $req = 'SELECT hs.`id_'.$this->tableName.'_slides` as id_slide
                FROM `'._DB_PREFIX_.$this->tableName  . '` hs
                WHERE hs.`id_'.$this->tableName.'_slides` = '.(int)$id_slide;
        $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($req);

        return ($row);
    }

    public function renderList()
    {
        $slides = $this->getSlides();
        foreach ($slides as $key => $slide) {
            $slides[$key]['status'] = $this->displayStatus($slide['id_slide'], $slide['active']);
            $associated_shop_ids = $this->getAssociatedIdsShop((int)$slide['id_slide']);
            if ($associated_shop_ids && count($associated_shop_ids) > 1) {
                $slides[$key]['is_shared'] = true;
            } else {
                $slides[$key]['is_shared'] = false;
            }
        }

        $this->context->smarty->assign(
            array(
                'link' => $this->context->link,
                'slides' => $slides,
                'image_baseurl' => $this->_path.'images/',
                'name' => $this->customName,
            )
        );

        return $this->display(__FILE__, 'list.tpl');
    }

    public function renderAddForm()
    {
        $idLang = $this->context->language->id;
        $effect_array = array();
        foreach ($this->effect_array as $key=>$effect) {
            $effect_array[$key] = array();
            $effect_array[$key]['id_transition'] = (int)$key;
            $effect_array[$key]['name'] = $effect;
        }

        //CMS::getCMSPages($idLang = null, $idCmsCategory = null, $active = true, $idShop = null);
        $cmsArr = CMS::getCMSPages($idLang);
        $cms_page_array = array();
        $cms_page_array[] = array(
            'id_cms' => 0,
            'name' => $this->l('none'),
        );
        foreach ($cmsArr as $key=>$cms) {
            $cms_page_array[] = array(
                'id_cms' => (int)$cms['id_cms'],
                'name' => $cms['meta_title'],
            );
        }

        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Block information'),
                    'icon' => 'icon-cogs',
                ),
                'description' => '<b>' . $this->displayName . ' ver. ' . $this->version . ' (dev. ' . $this->dev_version . ')</b>'
                                . '<br><ol><li>' . $this->l('Configure a custom block with image, description, title, effects and link.') . '</li>'
                                . '<li>' . $this->l('Download mxb_featuredproducts and use its TAG system to create blocks with sliders of products.')
                                . ' <a href="https://www.mxbinformatica.it/prestashop-mxb-featuredproducts/" target="_new">' . $this->l('Click here for more informations.') . '</a></li></ol>'
                                . '<br><b>' . $this->l('Documentation') . '</b>: <a href="https://www.mxbinformatica.it/prestashop-mxb-customblocks/" target="_blank">' . $this->l('click here') . '</a></li></ol>',
                'input' => array(
                    array(
                        'type' => 'file_lang',
                        'label' => $this->getTranslator()->trans('Image', array(), 'Admin.Global'),
                        'name' => 'image',
                        'required' => true,
                        'lang' => true,
                        'desc' => $this->getTranslator()->trans('Maximum image size: %s.', array(ini_get('upload_max_filesize')), 'Admin.Global') . ' - '
                                . $this->getTranslator()->trans('Image is optional.', array(), 'Admin.Global') . ' - '
                                . $this->getTranslator()->trans('The image will be used as block background. The block dimension will be the image dimension.', array(), 'Admin.Global')
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Remove image'),
                        'name' => 'remove_image_slide',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->getTranslator()->trans('Yes', array(), 'Admin.Global')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->getTranslator()->trans('No', array(), 'Admin.Global')
                            )
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Fixed background'),
                        'name' => 'fixed_bg_slide',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->getTranslator()->trans('Yes', array(), 'Admin.Global')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->getTranslator()->trans('No', array(), 'Admin.Global')
                            )
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Repeat background'),
                        'name' => 'repeat_bg_slide',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->getTranslator()->trans('Yes', array(), 'Admin.Global')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->getTranslator()->trans('No', array(), 'Admin.Global')
                            )
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Effect when shown'),
                        'name' => 'id_transition_slide',
                        'options' => array(
                            'query' => $effect_array,
                            'id' => 'id_transition',
                            'name' => 'name',
                        ),
                        'col' => '4',
                    ),

                    array(
                        'type' => 'text',
                        'label' => $this->l('Custom css class (you can use one or more bootstrap class)'),
                        'name' => 'custom_class_slide',
                        'desc' => array(
                                    $this->l('Create your own class and use custom CSS in main module settings. Set a min-height for your class to have different minimum height for every block.'),
                                    '<br><b>' . $this->l('Add usefull classes:') . '</b>'
                                    . '<ul>'
                                    . '<li><b><a class="mxb_add_custom_class" data-class="read-more" style="cursor: pointer;">add "read-more" class</a></b>'
                                    . ': ' . $this->l('add read-more class to limit the paragraph height and show a "read more" button.') . '</li>'
                                    . '<li><b><a class="mxb_add_custom_class" data-class="no-borders" style="cursor: pointer;">add "no-borders" class</a></b>'
                                    . ': ' . $this->l('no shadow and borders for this paragraph.') . '</li>'
                                    . '</ul>'
                                    . '<br>' . $this->l('Hide on some devices using bootstrap display class:')
                                    . '<br><b>' . $this->l('Bootstrap 4 and up') . '</b>'
                                    . '<ul><li>Hidden on all: <b><a class="mxb_add_custom_class" data-class="d-none" style="cursor: pointer;">d-none</a></b></li>'
                                    . '<li>Hidden on xs: <b><a class="mxb_add_custom_class" data-class="d-none d-sm-block" style="cursor: pointer;">d-none d-sm-block</a></b></li>'
                                    . '<li>Hidden on sm: <b><a class="mxb_add_custom_class" data-class="d-sm-none d-md-block" style="cursor: pointer;">d-sm-none d-md-block</a></b></li>'
                                    . '<li>Hidden on md: <b><a class="mxb_add_custom_class" data-class="d-md-none d-lg-block" style="cursor: pointer;">d-md-none d-lg-block</a></b></li>'
                                    . '<li>Hidden on lg: <b><a class="mxb_add_custom_class" data-class="d-lg-none d-xl-block" style="cursor: pointer;">d-lg-none d-xl-block</a></b></li>'
                                    . '<li>Hidden on xl: <b><a class="mxb_add_custom_class" data-class="d-xl-none" style="cursor: pointer;">d-xl-none</a></b></li></ul>'
                                    . '<br>'
                                    . '<b>' . $this->l('Bootstrap 4 alpha down') . '</b>'
                                    . '<ul><li>Hidden on all: <b><a class="mxb_add_custom_class" data-class="hidden-xs-up" style="cursor: pointer;">hidden-xs-up</a></b></li>'
                                    . '<li>Hidden on xs and smaller: <b><a class="mxb_add_custom_class" data-class="hidden-xs-down" style="cursor: pointer;">hidden-xs-down</a></b></li>'
                                    . '<li>Hidden on sm and smaller: <b><a class="mxb_add_custom_class" data-class="hidden-sm-down" style="cursor: pointer;">hidden-sm-down</a></b></li>'
                                    . '<li>Hidden on md and smaller: <b><a class="mxb_add_custom_class" data-class="hidden-md-down" style="cursor: pointer;">hidden-md-down</a></b></li>'
                                    . '<li>Hidden on lg and smaller: <b><a class="mxb_add_custom_class" data-class="hidden-lg-down" style="cursor: pointer;">hidden-lg-down</a></b></li>'
                                    . '<li>Hidden on xl and smaller: <b><a class="mxb_add_custom_class" data-class="hidden-xl-down" style="cursor: pointer;">hidden-xl-down</a></b></li></ul>'
                                    . '<br>'
                                    . '<b>' . $this->l('Margin and padding') . '</b>'
                                    . '<ul><li>Margin top: <b><a class="mxb_add_custom_class" data-class="mt-3" style="cursor: pointer;">mt-3</a></b></li>'
                                    . '<li>Margin bottom small: <b><a class="mxb_add_custom_class" data-class="mb-1" style="cursor: pointer;">mb-1</a></b></li>'
                                    . '<li>Margin top+bottom: <b><a class="mxb_add_custom_class" data-class="my-3" style="cursor: pointer;">my-3</a></b></li>'
                                    . '<li>Margin left+right: <b><a class="mxb_add_custom_class" data-class="mx-3" style="cursor: pointer;">mx-3</a></b></li>'
                                    . '<li>Padding small: <b><a class="mxb_add_custom_class" data-class="p-1" style="cursor: pointer;">p-1</a></b></li>'
                                    . '<li>Padding left+right: <b><a class="mxb_add_custom_class" data-class="px-3" style="cursor: pointer;">px-3</a></b></li>'
                                    . '<li>Padding top: <b><a class="mxb_add_custom_class" data-class="pt-3" style="cursor: pointer;">pt-3</a></b></li></ul>'
                                    . '(*) ' . $this->l('use "p" for padding, "m" for margin, followed by the direction: t=top, b=bottom, l=left, r=right, x=left and right, y=top and bottom. Third value after the "-" is the thickness (from 0 to 5). So "padding_top-small" is "pt-1" and "no padding left+right" is "px-0.')
                        ),
                    ),

                    array(
                        'type' => 'text',
                        'label' => $this->getTranslator()->trans('Title', array(), 'Admin.Global'),
                        'name' => 'title',
                        'lang' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Target URL'),
                        'name' => 'url',
                        'required' => false,
                        'lang' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Caption'),
                        'name' => 'legend',
                        'lang' => true,
                    ),
                    array(
                        'type' => 'textarea',
                        'label' => $this->getTranslator()->trans('Description', array(), 'Admin.Global'),
                        'name' => 'description',
                        'autoload_rte' => true,
                        'lang' => true,
                        'desc' => array(
                            '<strong>' . $this->l('Height of the block') . '</strong>: ',
                            '<ul><li>' . $this->l('When a title or description is set, the height of the background image will depend on it. To set a minimum block height, please use custom CSS in customblocks configuration and set a min-height value for the main div or the custom CSS class of this block (read above). Look at the custom.css sample for more informations.') . '</li></ul>',

                            '<strong>' . $this->l('Custom blocks slider') . '</strong>: ',
                            '<ul><li>' . $this->l('You can create a block slider that looks similar to the classic featured product slider, with free content into each element') . '</li>'
                                . '<li>' . $this->l('You can put your content in each slide: there are 5 different areas:')
                                . '<ul><li>header</li>'
                                . '<li>body</li>'
                                . '<li>image</li>'
                                . '<li>footer</li>'
                                . '<li>url</li></ul></li>'
                                . '<li>' . $this->l('The content must be set into the block textarea using the "code" view (first button [< >] on the left of the editor header), and it must list every slide using a very simple html structure. This is a sample for a slide with 2 simple elements:') . '</li>'
                                . '<br><textarea style="height: 250px;">'
                                    . '<div class="blockslider">' . PHP_EOL
                                    . '    <div class="blockslide">' . PHP_EOL
                                    . '        <div class="bsheader">Put here the title of the FIRST slide</div>' . PHP_EOL
                                    . '        <div class="bsbody">Put here your description</div>' . PHP_EOL
                                    . '        <div class="bsimage onTop">https://www.mysite.com/images/my-image.jpg</div>' . PHP_EOL
                                    . '        <div class="bslink">https://www.mysite.com/open-this-page.html</div>' . PHP_EOL
                                    . '        <div class="bsfooter">&raquo; click here to open the link above</div>' . PHP_EOL
                                    . '    </div>' . PHP_EOL . PHP_EOL
                                    . '    <div class="blockslide">' . PHP_EOL
                                    . '        <div class="bsheader">Put here the title of the SECOND slide</div>' . PHP_EOL
                                    . '        <div class="bsbody">Put here your description</div>' . PHP_EOL
                                    . '        <div class="bsimage onTop">https://www.mysite.com/images/my-second-image.jpg</div>' . PHP_EOL
                                    . '        <div class="bslink">https://www.mysite.com/open-this-other-page.html</div>' . PHP_EOL
                                    . '        <div class="bsfooter">&raquo; click here to open the link above</div>' . PHP_EOL
                                    . '    </div>' . PHP_EOL
                                    . '</div>' . PHP_EOL
                                . '</textarea><br><br></li>'

                            . '<li><strong>' . $this->l('Create blocks with category, manufacturer or cms content:') . '</strong> '
                                . '<br>' . $this->l('a specific div must be set into "blockslide" div. The div class will set the element type and its content will set the id of the element.')
                                . '<br>* ' . $this->l('the ID value can be set as "rnd" too: a random record will be load every time.')
                                . '<br>** ' . $this->l('you must set other values (header, body and image) with an empty content to show the loaded values correctly')
                                . '<br>*** ' . $this->l('for elements without image (i.e. cms, or a missing manufacturer image) you can set a static image url manually into the "bsimage" div.')
                                . '<br>' . $this->l('List of classes and possible values:')
                                    . '<ul><li>bscategory: id_category, list of IDs, "rnd"</li>'
                                    . '<li>bsmanufacturer: id_manufacturer, list of IDs, "rnd"</li>'
                                    . '<li>bsproduct: id_product, list of IDs, "cat[id_category]" to get a random product for a specific category, "man[id_manufacturer]" to get a random product for a specific brand</li>'
                                    . '<li>bscms: id_category, list of IDs, "rnd"</li></ul>'
                                . '<br><br>' . $this->l('For example:')
                                    . '<br><pre><code class="html">' . htmlentities('<div class="blockslider table">')
                                    . '<br>    ' . htmlentities('<div class="blockslide">')
                                    . '<br>        ' . htmlentities('<div class="bscategory">12</div>')
                                    . '<br>        ' . htmlentities('<div class="bsimage onTop"></div>')
                                    . '<br>        ' . htmlentities('<div class="bsheader"></div>')
                                    . '<br>        ' . htmlentities('<div class="bsbody"></div>')
                                    . '<br>        ' . htmlentities('<div class="bsurl"></div>')
                                    . '<br>        ' . htmlentities('<div class="bsfooter">Click here</div>')
                                    . '<br>    ' . htmlentities('</div>')
                                    . '<br>' . htmlentities('</div>')
                                    . '</code></pre>'
                            . '</li>'

                            . '<li><strong>' . $this->l('Create a static sequence of columns without slider:') . '</strong> ' . $this->l('add the class "table" to the first div:')
                                . '<br><code>' . htmlentities('<div class="blockslider table">')
                                . '</code><br><br></li>'

                            . '<li><strong>' . $this->l('Create single blocks with more than 1 product in it:') . '</strong> ' . $this->l('use the option "group=n"')
                                . '<ul><li>' . $this->l('use this option in junction with "table" option and for "bsproduct" blocks only') . '</li>'
                                . '<li>' . $this->l('into "params" div of a block with "bsproduct" option, add the row "group=4" where 4 is the number of products you want to show') . '</li>'
                                . '<li>' . $this->l('products will be shown in a 2 columns container with image, name and link to product page.') . '</li>'
                                . '</ul>'
                                . '<br><pre><code class="html">' . htmlentities('<div class="blockslider table">')
                                    . '<br>    ' . htmlentities('<div class="blockslide">')
                                    . '<br>        ' . htmlentities('<div class="bsparams">')
                                    . '<br>            group=4,'
                                . '</code></pre><br><br></li>'

                            . '<li><strong>' . $this->l('Blocks classes:') . '</strong> ' . $this->l('add custom or bootstrap classes to customize each block. For example you can hide some blocks for a specific resolution using " hidden-md-down". See the bootstrap classes list above.')
                            . '<br><code>' . htmlentities('<div class="blockslider table">')
                                . '<br>' . htmlentities('<div class="blockslide hidden-md-down">')
                                . '<br>..... '
                                . '<br>' . htmlentities('</div>')
                                . '</code><br><br></li>'

                            . '<li><strong>' . $this->l('Blocks aspect:') . '</strong> '
                                . '<ul><li>' . $this->l('CSS: use the general custom CSS to customize the look of the sliders: you will find instructions in the CSS template') . '</li>'
                                . '<li>' . $this->l('insert a "params" div listing all graphic preferences for a single block. Here the list of tags you can use:')
                                    . '<br>* ' . $this->l('for dimensions you omit "px" and write only the value')
                                    . '<br>** ' . $this->l('set the image position using the specific classess listed in the next point')
                                    . '<br>** ' . $this->l('use a comma (,) to separate values (width=120,height=240,margin=12px ...)')
                                    . '<ul><li>group: <i>' . $this->l('omit for "1 element" block. Set it to 2+ to get a block with more than one product. Use this option in "bsproduct" blocks only to show more than one random product for selecter categories or manufacturers. Use even numbers, try with blocks=4 first!') . '</i></li>'
                                    . '<li>bg-color: <i>' . $this->l('block background color: use hex format (#CACACA), color names (red, blue...) or rgb/rgba format: rgb(red, green, blue, alpha)') . '</i></li>'
                                    . '<li>margin: <i>' . $this->l('single block margin: dimensions must be in pixel and will set the four slide margin (top bottom left and right') . '</i></li>'
                                    . '<li>width: <i>' . $this->l('single block dimensions must be in pixel for slider, and can be in percentage for static columns ("table" option, above)') . '</i></li>'
                                    . '<li>height: <i>' . $this->l('as above') . '</i></li>'
                                    . '<li>body-width: <i>' . $this->l('dimensions of a slide block with the image option "onLeft" or "onRight". Dimensions can be in pixel or percentage, and their sum should be less than the slide width, considering margin and padding too') . '</i></li>'
                                    . '<li>body-height: <i>' . $this->l('as above') . '</i></li>'
                                    . '<li>body-align: <i>' . $this->l('text alignment: left, center, right') . '</i></li>'
                                    . '<li>body-color: <i>' . $this->l('text color: uuse hex, color names, rgb or rgba format') . '</i></li>'
                                    . '<li>header-align: <i>' . $this->l('title alignment: left, center, right') . '</i></li>'
                                    . '<li>header-color: <i>' . $this->l('title color: use hex, color names, rgb or rgba format') . '</i></li>'
                                    . '<li>image-width: <i>' . $this->l('dimension in pixel or percentage') . '</i></li>'
                                    . '<li>image-height: <i>' . $this->l('dimension in pixel') . '</i></li></ul>'

                                . '<br><pre><code class="html">' . htmlentities('<div class="blockslider table">')
                                    . '<br>    ' . htmlentities('<div class="blockslide">')
                                    . '<br>        ' . htmlentities('<div class="bsparams">')
                                    . '<br>            group=4,'
                                    . '<br>            bg-color=grey,'
                                    . '<br>            margin=12px,'
                                    . '<br>            width=240,'
                                    . '<br>            height=240,'
                                    . '<br>            body-width=130,'
                                    . '<br>            body-height=130,'
                                    . '<br>            body-align=center,'
                                    . '<br>            body-color=#000000,'
                                    . '<br>            header-align=center,'
                                    . '<br>            header-color=red,'
                                    . '<br>            image-width=100,'
                                    . '<br>            image-height=120'
                                    . '<br>        ' . htmlentities('</div>')
                                    . '<br>     ..... '
                                . '</code></pre></li></ul>'
                            . '<br><br></li>'

                            . '<li><strong>' . $this->l('Image position:') . '</strong> ' . $this->l('add one of the following classes to "blockside-image" class list:')
                                  . '<ul><li>onTop</li>'
                                  . '<li>onBottom</li>'
                                  . '<li>onLeft</li>'
                                  . '<li>onRight</li>'
                                  . '<li>asBackground</li></ul>'
                            . '<br><br></li>'

                            . '<li><strong>' . $this->l('CSS:') . '</strong> ' . $this->l('use the general custom CSS to customize the look of the sliders: you will find instructions in the CSS template.') . '<br><br></li>'

                            . '<li><strong>' . $this->l('Full block slider sample:') . '</strong> ' . $this->l('this is a full sample with many options that you can copy and paste into your block, modify with your data and check the results. Try to set and unset the "table" option to switch from a statica table and a slider.')
                                . '<br><textarea style="height: 250px;">'
                                    . '<div class="blockslider">' . PHP_EOL
                                    . '    <div class="blockslide">' . PHP_EOL
                                    . '        <div class="bsparams">' . PHP_EOL
                                    . '            margin=12px,' . PHP_EOL
                                    . '            width=240' . PHP_EOL
                                    . '        </div>' . PHP_EOL
                                    . '        <div class="bsheader">Prodotti green</div>' . PHP_EOL
                                    . '        <div class="bsbody">Riciclabilit&agrave;, certificazioni, brand, materiale e imballaggio vanno a costituire gli elementi di valutazione per abbinare la nostra icona "green" ad un determinato prodotto.</div>' . PHP_EOL
                                    . '        <div class="bsimage onTop">https://www.carrellopromozionale.it/img/cms/blocks/ecologic.png</div>' . PHP_EOL
                                    . '        <div class="bslink">https://www.carrellopromozionale.it/content/32-prodotti-ecologici</div>' . PHP_EOL
                                    . '        <div class="bsfooter">&raquo; leggi di pi&ugrave;</div>' . PHP_EOL
                                    . '    </div>' . PHP_EOL
                                    . PHP_EOL
                                    . '    <div class="blockslide">' . PHP_EOL
                                    . '        <div class="bsparams">' . PHP_EOL
                                    . '            margin=12px,' . PHP_EOL
                                    . '            width=240' . PHP_EOL
                                    . '        </div>' . PHP_EOL
                                    . '        <div class="bsheader">Certificazioni FSC</div>' . PHP_EOL
                                    . '        <div class="bsbody">Tutela ambientale e sfruttamento sostenibile delle risorse.</div>' . PHP_EOL
                                    . '        <div class="bsimage onBottom">https://www.carrellopromozionale.it/img/cms/blocks/fsc.png</div>' . PHP_EOL
                                    . '        <div class="bslink">https://www.carrellopromozionale.it/ricerca?controller=search&s=Fsc_certified</div>' . PHP_EOL
                                    . '        <div class="bsfooter">&raquo; vedi prodotti</div>' . PHP_EOL
                                    . '    </div>' . PHP_EOL
                                    . PHP_EOL
                                    . '    <div class="blockslide">' . PHP_EOL
                                    . '        <div class="bsparams">' . PHP_EOL
                                    . '            margin=12px,' . PHP_EOL
                                    . '            width=240,' . PHP_EOL
                                    . '            body-width=130,' . PHP_EOL
                                    . '            image-width=100' . PHP_EOL
                                    . '        </div>' . PHP_EOL
                                    . '        <div class="bsheader">World import</div>' . PHP_EOL
                                    . '        <div class="bsbody">Tecnologia, articoli per la casa, penne e scrittura. Importazione diretta, prezzi vantaggiosi e stampa inclusaper centinaia di idee regalo.</div>' . PHP_EOL
                                    . '        <div class="bsimage onLeft">https://www.carrellopromozionale.it/img/cms/blocks/world-import.png</div>' . PHP_EOL
                                    . '        <div class="bslink">https://www.carrellopromozionale.it/328-world-import</div>' . PHP_EOL
                                    . '        <div class="bsfooter">&raquo; vedi prodotti</div>' . PHP_EOL
                                    . '    </div>' . PHP_EOL
                                    . PHP_EOL
                                    . '    <div class="blockslide">' . PHP_EOL
                                    . '        <div class="bsparams">' . PHP_EOL
                                    . '            margin=12px,' . PHP_EOL
                                    . '            width=240,' . PHP_EOL
                                    . '            body-width=130,' . PHP_EOL
                                    . '            image-width=100' . PHP_EOL
                                    . '        </div>' . PHP_EOL
                                    . '        <div class="bsheader">Prodotti di tendenza</div>' . PHP_EOL
                                    . '        <div class="bsbody">Punta dritto all\'obiettivo: cosa piace di più alle aziende? Quali sono i prodotti più acquistati? Lasciati consigliare da una selezione di prodotti di tendenza: i nostri best sellers !</div>' . PHP_EOL
                                    . '        <div class="bsimage onRight">https://www.carrellopromozionale.it/img/cms/blocks/bestsellers.png</div>' . PHP_EOL
                                    . '        <div class="bslink">https://www.carrellopromozionale.it/ricerca?controller=search&s=Bestsellers</div>' . PHP_EOL
                                    . '        <div class="bsfooter">&raquo; vedi prodotti</div>' . PHP_EOL
                                    . '    </div>' . PHP_EOL
                                    . PHP_EOL
                                    . '    <div class="blockslide">' . PHP_EOL
                                    . '        <div class="bsparams">' . PHP_EOL
                                    . '            margin=12px,' . PHP_EOL
                                    . '            width=240,' . PHP_EOL
                                    . '            header-align=center,' . PHP_EOL
                                    . '            body-align=center,' . PHP_EOL
                                    . '            body-color=#000000,' . PHP_EOL
                                    . '            header-color=red' . PHP_EOL
                                    . '        </div>' . PHP_EOL
                                    . '        <div class="bsheader">Immagine asBackground</div>' . PHP_EOL
                                    . '        <div class="bsbody">Usa i CSS per personalizzare al meglio l\'aspetto dei riquadri</div>' . PHP_EOL
                                    . '        <div class="bsimage asBackground">https://www.carrellopromozionale.it/img/cms/blocks/bestsellers.png</div>' . PHP_EOL
                                    . '        <div class="bslink">https://www.carrellopromozionale.it/ricerca?controller=search&s=Bestsellers</div>' . PHP_EOL
                                    . '        <div class="bsfooter">&raquo; scopri</div>' . PHP_EOL
                                    . '    </div>' . PHP_EOL
                                    . PHP_EOL
                                    . '    <div class="blockslide">' . PHP_EOL
                                    . '        <div class="bsbrand">rnd</div>' . PHP_EOL
                                    . '        <div class="bsparams">' . PHP_EOL
                                    . '            margin=12px,' . PHP_EOL
                                    . '            width=240,' . PHP_EOL
                                    . '            header-align=center,' . PHP_EOL
                                    . '            body-align=center,' . PHP_EOL
                                    . '            header-color=red,' . PHP_EOL
                                    . '            body-color=#000000' . PHP_EOL
                                    . '        </div>' . PHP_EOL
                                    . PHP_EOL
                                    . '        <div class="bsheader">manufacturer rnd</div>' . PHP_EOL
                                    . '        <div class="bsbody">descrizione</div>' . PHP_EOL
                                    . '        <div class="bsimage onTop"></div>' . PHP_EOL
                                    . '        <div class="bslink"></div>' . PHP_EOL
                                    . '        <div class="bsfooter">&raquo; vedi prodotti</div>' . PHP_EOL
                                    . '    </div>' . PHP_EOL
                                    . PHP_EOL
                                    . '    <div class="blockslide">' . PHP_EOL
                                    . '        <div class="bscategory">12</div>' . PHP_EOL
                                    . '        <div class="bsparams">' . PHP_EOL
                                    . '            margin=12px,' . PHP_EOL
                                    . '            width=240,' . PHP_EOL
                                    . '            header-align=center,' . PHP_EOL
                                    . '            body-align=center,' . PHP_EOL
                                    . '            header-color=red,' . PHP_EOL
                                    . '            body-color=#000000' . PHP_EOL
                                    . '        </div>' . PHP_EOL
                                    . '        <div class="bsheader">categoria 12</div>' . PHP_EOL
                                    . '        <div class="bsbody">descrizione</div>' . PHP_EOL
                                    . '        <div class="bsimage asBackground"></div>' . PHP_EOL
                                    . '        <div class="bslink"></div>' . PHP_EOL
                                    . '        <div class="bsfooter">&raquo; vedi prodotti</div>' . PHP_EOL
                                    . '    </div>' . PHP_EOL
                                    . PHP_EOL
                                    . '    <div class="blockslide">' . PHP_EOL
                                    . '        <div class="bscms">5</div>' . PHP_EOL
                                    . '        <div class="bsparams">' . PHP_EOL
                                    . '            margin=12px,' . PHP_EOL
                                    . '            width=240,' . PHP_EOL
                                    . '            header-align=center,' . PHP_EOL
                                    . '            body-align=center,' . PHP_EOL
                                    . '            header-color=red,' . PHP_EOL
                                    . '            body-color=#000000' . PHP_EOL
                                    . '        </div>' . PHP_EOL
                                    . '        <div class="bsheader">cms 5</div>' . PHP_EOL
                                    . '        <div class="bsbody">descrizione</div>' . PHP_EOL
                                    . '        <div class="bsimage onTop"></div>' . PHP_EOL
                                    . '        <div class="bslink"></div>' . PHP_EOL
                                    . '        <div class="bsfooter">&raquo; approfondisci</div>' . PHP_EOL
                                    . '    </div>' . PHP_EOL
                                    . '</div>' . PHP_EOL
                                . '</textarea><br><br></li>'
                            . '</ul>',

                            '<strong>' . $this->l('If you have "mxb_featureproducts" module installed') . '</strong>: ',
                                '<ul><li>' . $this->l('a "mxb_featureproducts tag string" can be put directly into a block description to obtain various blocks showing different products as a horizontally scrollable banners.') . '</li>'
                                . '<li>' . $this->l('Sample of tag usage (see mxb_featuredproducts help for full instructions)')
                                . '<br><code>[mxb_featuredproducts_one products=4, category=2, brand=0, random=1, position=here, slide=1, footer-link=0, filter-text=cotton, best-sales=0, class=my-custom-class, title=Intresting products!]</code></li>'
                                . '<li>' . $this->l('Note: using product tags or your custom HTML, I suggest to avoid background image. Remove it if already uploaded enabling the option "remove image" and saving the page.') . '</li></ul>',

                            '<strong>' . $this->l('Animations') . '</strong>: ',
                            '<ul><li>' . $this->l('You can surround one or more paragraph with a div giving it a class named as one of the transition in the above list to make it move.') . '</li>'
                                . '<li>' . $this->l('Please use the [<>] button of the text editor to edit as html ad add a div like this sample:')
                                . '<br><code>' . htmlentities('<div class="mxb-transition-slide-from-left"><p> ... text or other ... </p></div>"') . '</code></li></ul>',
                        ),
                    ),

                    /*
                    id_cms
                    */

                    array(
                        'type' => 'select',
                        'label' => $this->l('Show CMS Page'),
                        'name' => 'id_cms_slide',
                        'options' => array(
                            'query' => $cms_page_array,
                            'id' => 'id_cms',
                            'name' => 'name',
                        ),
                        'col' => '4',
                        'desc' => array(
                            $this->l('Select a CMS page to be shown into this block'),
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->getTranslator()->trans('Enabled', array(), 'Admin.Global'),
                        'name' => 'active_slide',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->getTranslator()->trans('Yes', array(), 'Admin.Global')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->getTranslator()->trans('No', array(), 'Admin.Global')
                            )
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->getTranslator()->trans('Save', array(), 'Admin.Actions'),
                )
            ),
        );

        if (Tools::isSubmit('id_slide') && $this->slideExists((int)Tools::getValue('id_slide'))) {
            $slide = $this->getSlide((int)Tools::getValue('id_slide'));
            $fields_form['form']['input'][] = array('type' => 'hidden', 'name' => 'id_slide');
            $fields_form['form']['images'] = $slide->image;

            $has_picture = true;

            foreach (Language::getLanguages(false) as $lang) {
                if (!isset($slide->image[$lang['id_lang']])) {
                    $has_picture &= false;
                }
            }

            if ($has_picture) {
                $fields_form['form']['input'][] = array('type' => 'hidden', 'name' => 'has_picture');
            }
        }

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->module = $this;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitSlide';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $language = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->tpl_vars = array(
            'base_url' => $this->context->shop->getBaseURL(),
            'language' => array(
                'id_lang' => $language->id,
                'iso_code' => $language->iso_code
            ),
            'fields_value' => $this->getAddFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'image_baseurl' => $this->_path.'images/'
        );

        $helper->override_folder = '/';

        $languages = Language::getLanguages(false);

        if (count($languages) > 1) {
            return $this->getMultiLanguageInfoMsg() . $helper->generateForm(array($fields_form));
        } else {
            return $helper->generateForm(array($fields_form));
        }
    }

    public function renderForm()
    {

        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->getTranslator()->trans('Settings', array(), 'Admin.Global'),
                    'icon' => 'icon-cogs'
                ),
                'description' => '<b>' . $this->displayName . ' ver. ' . $this->version . ' (dev. ' . $this->dev_version . ')</b>'
                                . '<br>' . $this->description
                                . '<br><ol><li>' . $this->l('Configure how many custom blocks you wish. Here you can set main settings and customize blocks appearence with CSS.') . '</li>'
                                . '<li>' . $this->l('Download mxb_featuredproducts and use its TAG system to create blocks with sliders of products.')
                                . ' <a href="https://www.mxbinformatica.it/prestashop-mxb-featuredproducts/" target="_new">' . $this->l('Click here for more informations.') . '</a></li></ol>'
                                . '<br><b>' . $this->l('Documentation') . '</b>: <a href="https://www.mxbinformatica.it/prestashop-mxb-customblocks/" target="_new">' . $this->l('click here') . '</a></li></ol>',
                'input' => array(
                    array(
                        'type' => 'text',
                        'label' => $this->l('Space between blocks'),
                        'name' => $this->CUSTOMBLOCKS_MARGIN,
                        'suffix' => 'px',
                        'class' => 'fixed-width-sm',
                        'desc' => $this->l('The bottom margin of every block.')
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('"Read more" height'),
                        'name' => $this->CUSTOMBLOCKS_READ_MORE_HEIGHT,
                        'suffix' => 'px',
                        'class' => 'fixed-width-sm',
                        'desc' => $this->l('The max height for paragraphs with "read-more" class.')
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Effect on scroll'),
                        'name' => $this->CUSTOMBLOCKS_EFFECT_ON_SCROLL,
                        'desc' => $this->l('Enable fade effect when element is being displayed in page.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->getTranslator()->trans('Enabled', array(), 'Admin.Global')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->getTranslator()->trans('Disabled', array(), 'Admin.Global')
                            )
                        ),
                    ),

                    array(
                        'type' => 'switch',
                        'label' => $this->l('Random blocks order'),
                        'name' => $this->CUSTOMBLOCKS_SHUFFLE_ENABLE,
                        'desc' => $this->l('Enable to display blocks with a random order.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->getTranslator()->trans('Enabled', array(), 'Admin.Global')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->getTranslator()->trans('Disabled', array(), 'Admin.Global')
                            )
                        ),
                    ),

                    array(
                        'type' => 'switch',
                        'label' => $this->l('Enable custom css'),
                        'name' => $this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE,
                        'desc' => $this->l('Enable custom css in the textarea below. Customize your blocks appearance.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->getTranslator()->trans('Enabled', array(), 'Admin.Global')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->getTranslator()->trans('Disabled', array(), 'Admin.Global')
                            )
                        ),
                    ),


                    array(
                        'type' => 'textarea',
                        'lang' => false,
                        'label' => $this->l('Custom CSS for blocks'),
                        'name' => $this->CUSTOMBLOCKS_CUSTOM_CSS_CONTENT,
                        'desc' => $this->l('You can modify this precompiled css to change colors, borders, shadows and so on, only for this module.')
                                            . '<br><br><b>' . $this->l('Download DEFAULT CSS file')
                                            . ': <a href="' . $this->getModulePathUri() . 'views/css/custom.css" target="_new">custom.css</a></b>'
                                            //. ': <a href="' . $this->context->link->getPageLink('',true) . 'modules/' . $this->name . '/css/converted/_customblocks_custom.css" target="_new"> _customblocks_custom.css</a></b>'
                    ),


                ),
                'submit' => array(
                    'title' => $this->getTranslator()->trans('Save', array(), 'Admin.Actions'),
                )
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitSlider';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }

    public function getConfigFieldsValues($forSmarty = false)
    {
        $id_shop_group = Shop::getContextShopGroupID();
        $id_shop = Shop::getContextShopID();

        $fields = array(
            $this->CUSTOMBLOCKS_MARGIN => Tools::getValue($this->CUSTOMBLOCKS_MARGIN, Configuration::get($this->CUSTOMBLOCKS_MARGIN, null, $id_shop_group, $id_shop)),
            $this->CUSTOMBLOCKS_EFFECT_ON_SCROLL => Tools::getValue($this->CUSTOMBLOCKS_EFFECT_ON_SCROLL, Configuration::get($this->CUSTOMBLOCKS_EFFECT_ON_SCROLL, null, $id_shop_group, $id_shop)),
            $this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE => Tools::getValue($this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE, Configuration::get($this->CUSTOMBLOCKS_CUSTOM_CSS_ENABLE, null, $id_shop_group, $id_shop)),
            $this->CUSTOMBLOCKS_SHUFFLE_ENABLE => Tools::getValue($this->CUSTOMBLOCKS_SHUFFLE_ENABLE, Configuration::get($this->CUSTOMBLOCKS_SHUFFLE_ENABLE, null, $id_shop_group, $id_shop)),
            $this->CUSTOMBLOCKS_READ_MORE_HEIGHT => Tools::getValue($this->CUSTOMBLOCKS_READ_MORE_HEIGHT, Configuration::get($this->CUSTOMBLOCKS_READ_MORE_HEIGHT, null, $id_shop_group, $id_shop)),
        );

        // get custom css
        $custom_css = $this->readCssFile();
        $fields[$forSmarty ? 'CUSTOMBLOCKS_CUSTOM_CSS_CONTENT' : $this->CUSTOMBLOCKS_CUSTOM_CSS_CONTENT] = Tools::getValue($this->CUSTOMBLOCKS_CUSTOM_CSS_CONTENT, $custom_css);

        return $fields;
    }

    public function getAddFieldsValues()
    {
        $fields = array();

        if (Tools::isSubmit('id_slide') && $this->slideExists((int)Tools::getValue('id_slide'))) {
            $slide = $this->getSlide((int)Tools::getValue('id_slide'));
            $fields['id_slide'] = (int)Tools::getValue('id_slide', $slide->id);
        } else {
            $slide = $this->getSlide();
        }


        $fields['active_slide'] = Tools::getValue('active_slide', $slide->active);
        $fields['fixed_bg_slide'] = Tools::getValue('fixed_bg_slide', $slide->fixed_bg);
        $fields['repeat_bg_slide'] = Tools::getValue('repeat_bg_slide', $slide->repeat_bg);
        $fields['id_transition_slide'] = Tools::getValue('id_transition_slide', $slide->id_transition);
        $fields['custom_class_slide'] = Tools::getValue('custom_class_slide', $slide->custom_class);
        $fields['id_cms_slide'] = Tools::getValue('id_cms_slide', $slide->id_cms);




        $fields['has_picture'] = true;
        $fields['remove_image_slide'] = 0;

        $languages = Language::getLanguages(false);

        foreach ($languages as $lang) {
            $fields['image'][$lang['id_lang']] = Tools::getValue('image_'.(int)$lang['id_lang']);
            $fields['title'][$lang['id_lang']] = Tools::getValue('title_'.(int)$lang['id_lang'], $slide->title[$lang['id_lang']]);
            $fields['url'][$lang['id_lang']] = Tools::getValue('url_'.(int)$lang['id_lang'], $slide->url[$lang['id_lang']]);
            $fields['legend'][$lang['id_lang']] = Tools::getValue('legend_'.(int)$lang['id_lang'], $slide->legend[$lang['id_lang']]);
            $fields['description'][$lang['id_lang']] = Tools::getValue('description_'.(int)$lang['id_lang'], $slide->description[$lang['id_lang']]);
        }

        return $fields;
    }

    protected function getMultiLanguageInfoMsg()
    {
        return '<p class="alert alert-warning">'.
                    $this->l('Since multiple languages are activated on your shop, please mind to upload your image for each one of them').
                '</p>';
    }

    protected function getWarningMultishopHtml()
    {
        if (Shop::getContext() == Shop::CONTEXT_GROUP || Shop::getContext() == Shop::CONTEXT_ALL) {
            return '<p class="alert alert-warning">' .
            $this->l('You cannot manage slides items from a "All Shops" or a "Group Shop" context, select directly the shop you want to edit') .
            '</p>';
        } else {
            return '';
        }
    }

    protected function getShopContextError($shop_contextualized_name, $mode)
    {
        if (is_array($shop_contextualized_name)) {
            $shop_contextualized_name = implode('<br/>', $shop_contextualized_name);
        }

        if ($mode == 'edit') {
            return '<p class="alert alert-danger">' .
                $this->l('You can only edit this slide from the shop(s) context') . ': ' .  $shop_contextualized_name .
                '</p>';
        } else {
            return '<p class="alert alert-danger">' .
                $this->l('You cannot add slides from a "All Shops" or a "Group Shop" context') .
                '</p>';
        }
    }

    protected function getShopAssociationError($id_slide)
    {
        return '<p class="alert alert-danger">'.
                        $this->l('Unable to get slide shop association information') . ' (id_slide: ' . (int)$id_slide . ')' .
                '</p>';
    }


    protected function getCurrentShopInfoMsg()
    {
        $shop_info = null;

        if (Shop::isFeatureActive()) {
            if (Shop::getContext() == Shop::CONTEXT_SHOP) {
                $shop_info = $this->l('The modifications will be applied to shop:') . ' ' . $this->context->shop->name;
            } else if (Shop::getContext() == Shop::CONTEXT_GROUP) {
                $shop_info = $this->l('The modifications will be applied to this group:') . ' ' . Shop::getContextShopGroup()->name;
            } else {
                $shop_info = $this->l('The modifications will be applied to all shops and shop groups');
            }

            return '<div class="alert alert-info">'.
                        $shop_info.
                    '</div>';
        } else {
            return '';
        }
    }

    protected function getSharedSlideWarning()
    {
        return '<p class="alert alert-warning">'.
            $this->l('This slide is shared with other shops! All shops associated to this slide will apply modifications made here').
            '</p>';
    }

    public function installModuleFiles()
    {
        $this->installModuleFile('translations/it.php');
        $this->installModuleFile('views/templates/hook/list.tpl');
        $this->installModuleFile('views/js/front.js');
        // $this->installModuleFile('views/css/front.css');
        $this->installModuleFile('views/css/custom.css', false);

        return true;
    }



    public function installModuleFile($module_file = 'translations/it.php', $overwrite = true)
    {
        $filedir = _PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR;
        $filename_src = $filedir . $module_file . '_'; // add underscore to open the source unmodifyed file
        $filename_tgt = $filedir . $module_file;
        $searchString = '_' . $this->rootName;

        if (!file_exists($filename_src)) {
            return false;
        }
        if (file_exists($filename_tgt) && !$overwrite) {
            return false;
        }
        $myfile = fopen($filename_src, "r");
        $filesize = filesize($filename_src);
        if ((int)$filesize === 0) {
            return false;
        }
        $content = fread($myfile, $filesize);
        fclose($myfile);

        // change default class with custom class and save file
        // text to replace not found ?
        // if (strpos($content, $searchString) === false)
        //     return true;

        // already replaced (replace text is already present) ?
        if (strpos($content, $this->name) === false) {
            $content = str_replace($searchString, $this->name, $content);
        }

        // update the real file
        $myfile = fopen($filename_tgt, "w");
        fwrite($myfile, $content);
        fclose($myfile);

        return true;
    }

    protected function saveCssFile($content, $cssfile = 'custom.css')
    {
        $filename = _PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.$this->customCssFolder.DIRECTORY_SEPARATOR . $cssfile;
        $myfile = fopen($filename, "w");
        fwrite($myfile, $content);
        fclose($myfile);
    }

    protected function readCssFile($cssfile = 'custom.css')
    {
        // check if file exists and create it if necessary
        $filename = _PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.$this->customCssFolder.DIRECTORY_SEPARATOR . $cssfile;
        if (!file_exists($filename)) {
            $this->installModuleFile('views'.DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR.$cssfile, false);
            copy(_PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR . $cssfile, $filename);
        }
        $myfile = fopen($filename, "r");
        $content = fread($myfile, filesize($filename));
        fclose($myfile);
        return $content;
    }

    public function getModulePathUri()
    {
        // $uri =  Tools::getHttpHost(true).__PS_BASE_URI__ . DIRECTORY_SEPARATOR . 'modules' . DIRECTORY_SEPARATOR . $this->name . DIRECTORY_SEPARATOR;    // first method: OK
        $uri = $this->path2url($this->getModulePath()); // second method: OK
        if (substr($uri, -1) != DIRECTORY_SEPARATOR) {
            $uri .= DIRECTORY_SEPARATOR;
        }
        return $uri;
    }

    public function getModulePath()
    {
        return _PS_MODULE_DIR_ . $this->name . DIRECTORY_SEPARATOR;
    }

    public function path2url($file, $Protocol=null)
    {
        if ($Protocol === null) {
            $Protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . '://';
        }
        // MxbToolsDebug::WriteLog('mxb_tools', 'Mxb_ToolsUtils::path2url', 'file: '.$file.', HTTP_HOST:'.$_SERVER['HTTP_HOST'].', DOCUMENT_ROOT:'.$_SERVER['DOCUMENT_ROOT']);
        $DOCUMENT_ROOT = $_SERVER['DOCUMENT_ROOT'];
        if (substr($DOCUMENT_ROOT, -1) != DIRECTORY_SEPARATOR) {
            $DOCUMENT_ROOT .= DIRECTORY_SEPARATOR;
        }
        $HTTP_HOST = $_SERVER['HTTP_HOST'];
        if (substr($HTTP_HOST, -1) != DIRECTORY_SEPARATOR) {
            $HTTP_HOST .= DIRECTORY_SEPARATOR;
        }

        return $Protocol.$HTTP_HOST.str_replace($DOCUMENT_ROOT, '', $file);
    }

    // function to delete all files and subfolders from folder
    public function emptyFolder($dir, $removeFolder = false) {
        if ( !file_exists( $dir ) )
            return;
        $structure = glob(rtrim($dir, "/").'/*');
        if (is_array($structure)) {
            foreach($structure as $file) {
                if (is_dir($file)) {
                    $this->emptyFolder($file,true);
                } elseif (is_file($file)) {
                    unlink($file);
                }
            }
        }
        if($removeFolder) {
            rmdir($dir);
        }
    }

    protected function checkForUpdates()
    {
        // check for back-office
        //if (!Tools::getIsset('controller'))
        //    return false;
        $version = Configuration::getGlobalValue($this->name . '_VERSION');
        if ($version === 0) {
            // module was fresh of installation: no updates are needed: update current version and exit
            Configuration::updateGlobalValue($this->name . '_VERSION', self::MXB_DEV_VERSION);
            if ((int)self::MXB_DEV_VERSION <= 1) {
                return false;
            }
        }
        if ((int)self::MXB_DEV_VERSION <= (int)$version) {
            return false;
        }

        $confirmations = array();
        $errors = array();

        for ($i = $version+1; $i <= self::MXB_DEV_VERSION; $i++) {
            $version = $i;
            switch ($version) {
                case 1:
                    break;
                case 2:
                    $this->installModuleFile('views/js/front.js');
                    // $this->installModuleFile('views/css/front.css');
                    break;
            }
        }
        // update current version
        Configuration::updateGlobalValue($this->name . '_VERSION', $version);
        return true;
    }
}
